package routers

import (
	"errors"
	"fmt"
	"time"

	baapi "3g3e.com/rechy/ck.ba.api"
	jwt "3g3e.com/rechy/ck.ba.api/jwt-v2"
	"3g3e.com/zmy/ck.platform.shop/api"
	"3g3e.com/zmy/ck.platform.shop/keys"
	"github.com/gin-gonic/gin"
)

const (
	JwtRealm         = "CK"
	JwtTokenHeadName = "ChangKe"
)

var identityKey = "id"

func getJwtAuth() (*jwt.GinJWTMiddleware, error) {
	authMiddleware, err := jwt.New(&jwt.GinJWTMiddleware{
		Realm:            JwtRealm,
		SigningAlgorithm: baapi.JwtSigningAlgorithm,
		Timeout:          30 * time.Minute,
		MaxRefresh:       20 * time.Minute,
		IdentityKey:      identityKey,

		// 从data一个用户里找个唯一值做token存储的id
		PayloadFunc: func(data interface{}) jwt.MapClaims {
			if v, ok := data.(*baapi.ModAuthor); ok {
				m := jwt.MapClaims{
					"author":    v.String(),
					identityKey: fmt.Sprintf("%v-%v", v.Type, v.ID),
				}
				return m
			}
			return jwt.MapClaims{}
		},

		// 从token存储的id值, 再读取出用户的相关账户信息
		IdentityHandler: func(c *gin.Context) interface{} {
			claims := jwt.ExtractClaims(c)
			author, _ := claims["author"].(string)
			return baapi.NewModAuthor([]byte(author))
		},

		// 登录时验证账号密码是否正常, 返回用户信息, err为nil就是登录成功
		Authenticator: func(c *gin.Context) (interface{}, time.Duration, error) {
			return nil, time.Hour * 24 * 3, errors.New("this is ba client")
		},

		// 这个方法就是认证当前页面是否有权操作
		Authorizator: func(data interface{}, c *gin.Context) bool {
			if v, ok := data.(*baapi.ModAuthor); ok {
				return v.HasAuth(c.Request)
			}

			return false
		},

		// 无权操作, 格式化回复内容
		Unauthorized: api.Unauthorized,

		// 登录返回
		LoginResponse: api.LoginResponse,

		// 出返回
		LogoutResponse: api.LogoutResponse,

		ModifyTokenHeader: modifyTokenHeader,
		KeyFuncVerify:     keyFuncVerify,
		KeyFuncSign:       keyFuncSign,

		TokenLookup:   "header: Authorization, query: token, cookie: jwt",
		TokenHeadName: JwtTokenHeadName,
		TimeFunc:      time.Now,
		SendCookie:    false,
	})

	if err != nil {
		return nil, err
	}

	errInit := authMiddleware.MiddlewareInit()
	if errInit != nil {
		return nil, errInit
	}

	return authMiddleware, nil
}

func modifyTokenHeader(header map[string]interface{}) map[string]interface{} {
	kmgr := keys.GetKeyManager()
	return kmgr.ModifyTokenHeader(header)
}

func keyFuncVerify(header map[string]interface{}) (interface{}, error) {
	kmgr := keys.GetKeyManager()
	return kmgr.KeyFuncVerify(header)
}

func keyFuncSign(header map[string]interface{}) (interface{}, error) {
	kmgr := keys.GetKeyManager()
	return kmgr.KeyFuncSign(header)
}
